package org.fuseri.moduleexercise.exercise;

import org.fuseri.model.dto.common.Result;
import org.fuseri.moduleexercise.common.DomainRepositoryImpl;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
 * An implementation of the ExerciseRepository interface
 * Provides access to Exercise entities stored in a data source
 */
@Repository
public class ExerciseRepositoryImpl extends DomainRepositoryImpl<Exercise> implements ExerciseRepository {

    /**
     * Filters the exercises by the specified difficulty level and course id,
     * and returns a {@link Result} object containing these filtered exercises
     * along with pagination information
     *
     * @param pageRequest the pagination settings for the result
     * @param courseId    the id of the course to filter by
     * @param difficulty  the difficulty level to filter by
     * @return a {@link Result} object containing a list of paginated exercises that match the filter criteria
     */
    @Override
    public Page<Exercise> filterPerDifficultyPerCourse(PageRequest pageRequest, String courseId, int difficulty) {

        int startIndex = pageRequest.getPageNumber() * pageRequest.getPageSize();

        List<Exercise> pageEntities = getItems().stream()
                .filter(e -> e.getCourseId().equals(courseId) && e.getDifficulty() == difficulty)
                .skip(startIndex)
                .limit(pageRequest.getPageSize())
                .toList();

        return new PageImpl<>(pageEntities, pageRequest, pageEntities.size());
    }
}
